-- schemas.sql
-- Create all database schemas for the data warehouse

CREATE SCHEMA IF NOT EXISTS staging;
CREATE SCHEMA IF NOT EXISTS analytics;
CREATE SCHEMA IF NOT EXISTS reports;
CREATE SCHEMA IF NOT EXISTS outlets;
CREATE SCHEMA IF NOT EXISTS mappings;
