
from mpcaHydro import data_manager_functions as dmf
from mpcaHydro import wiski

station_ids = ['E67013001']


dmf.download_wiski_data(con,station_ids,start_year = 2010,)



station_ids = ['H67009001',
 'H67012001',
 'H67013001',
 'H67014001',
 'H67016001',
 'H67014002',
 'E67013001',
 'W67036001',
 'W67004001']